#### ::: utils for DNABERT-viz motif search ::: ####

import os
import pandas as pd
import numpy as np
from Bio import Entrez
import json
from typing import List

import run_adni_with_dnabert

def kmer2seq(kmers):
    """
    Convert kmers to original sequence

    Arguments:
    kmers -- str, kmers separated by space.

    Returns:
    seq -- str, original sequence.

    """
    kmers_list = kmers.split(" ")
    bases = [kmer[0] for kmer in kmers_list[0:-1]]
    bases.append(kmers_list[-1])
    seq = "".join(bases)
    assert len(seq) == len(kmers_list) + len(kmers_list[0]) - 1
    return seq


def seq2kmer(seq, k):
    """
    Convert original sequence to kmers

    Arguments:
    seq -- str, original sequence.
    k -- int, kmer of length k specified.

    Returns:
    kmers -- str, kmers separated by space

    """
    kmer = [seq[x:x+k] for x in range(len(seq)+1-k)]
    kmers = " ".join(kmer)
    return kmers


def chr_locs_to_rsIDs(chrom_pos_list: List[str]):
    """
    Code to convert chromosome position to rsID. Adapted from code provided by a labmate.
    
    The format for chrom_pos_list's items are <chr_num>:<location>, e.g. 19:45412040.
    """
    Entrez.email = '<EMAIL>'

    rsid_list = []

    for cp in chrom_pos_list:
        c = cp.split(':')[0]
        p = cp.split(':')[1]

        handle = Entrez.esearch(db='snp', term='({}[Chromosome]) AND {}[Base Position Previous]'.format(c, p), retmode='json')
        json_res = json.loads(handle.read().strip())
        ids = json_res['esearchresult']['idlist']
        handle.close()

        if len(ids) == 0:
            rsid_list.append("no SNP found")
        else:
            rsid_list.append("rs" + str(ids[0]))

    return rsid_list


def convert_snp_pos_to_chrom_pos(snp_pos_list: List[str], gene_name: str, gene_annot_filename: str) -> List[str]:
    chrom_pos_list = []
    chrom_pos_list_no_chr_num = []
    gene_start_pos = run_adni_with_dnabert.get_gene_location(gene_name, gene_annot_filename)[0]
    for snp_pos in snp_pos_list:
        new_location = gene_start_pos + int(snp_pos) + 1 # add 1 to account for off by one
        chrom_pos_list.append("19:" + str(new_location))
        chrom_pos_list_no_chr_num.append(new_location)

    return chrom_pos_list, chrom_pos_list_no_chr_num


def add_rsIDs_to_SNP_data(gene_annot_filename: str, snp_pos_filename: str, output_filename: str, gene_name: str) -> None:
    df = pd.read_csv(snp_pos_filename)
    snp_pos_list = df[df.columns[0]].values.tolist() 
    chrom_pos_list, chrom_pos_list_no_chr_num = convert_snp_pos_to_chrom_pos(snp_pos_list, gene_name, gene_annot_filename)
    rsid_list = chr_locs_to_rsIDs(chrom_pos_list)
    df.insert(0, "Location in chr19", chrom_pos_list_no_chr_num)
    df.insert(0, "rsID", rsid_list)
    df = df.drop(columns="SNP location within gene")
    df.to_csv(output_filename, index=False)


def main():
    GENE_ANNOT_FILENAME = "/home/user/Alzheimers-genetics-NNs/gene_annot.csv"
    gene_list = ["APOE", "EXOC3L2", "SIGLEC11", "TOMM40"]
    for gene in gene_list:
        # results without downsampling
        snp_pos_filename = "/home/user/svm_results/{}_svm_results_by_col.csv".format(gene)
        output_filename = snp_pos_filename.strip(".csv") + "_with_rsID.csv"
        add_rsIDs_to_SNP_data(GENE_ANNOT_FILENAME, snp_pos_filename, output_filename, gene)

        # results with downsampling
        snp_pos_filename_downsampled = "/home/user/svm_results/{}_svm_results_by_col_downsample.csv".format(gene)
        output_filename_downsampled = snp_pos_filename_downsampled.strip(".csv") + "_with_rsID.csv"
        add_rsIDs_to_SNP_data(GENE_ANNOT_FILENAME, snp_pos_filename_downsampled, output_filename_downsampled, gene)


if __name__ == "__main__":
    main()